@@ -326,9 +326,7 @@ class Attention(nn.Module):
         *,
         dim_head = 64,
         heads = 8,
-        dropout = 0.,
         causal = False,
-        rotary_emb = None
     ):
         super().__init__()
         self.scale = dim_head ** -0.5
@@ -337,14 +335,11 @@ class Attention(nn.Module):
 
         self.causal = causal
         self.norm = LayerNorm(dim)
-        self.dropout = nn.Dropout(dropout)
 
         self.null_kv = nn.Parameter(torch.randn(2, dim_head))
         self.to_q = nn.Linear(dim, inner_dim, bias = False)
         self.to_kv = nn.Linear(dim, dim_head * 2, bias = False)
 
-        self.rotary_emb = rotary_emb
-
         self.to_out = nn.Sequential(
             nn.Linear(inner_dim, dim, bias = False),
             LayerNorm(dim)
@@ -359,11 +354,6 @@ class Attention(nn.Module):
         q = rearrange(q, 'b n (h d) -> b h n d', h = self.heads)
         q = q * self.scale
 
-        # rotary embeddings
-
-        if exists(self.rotary_emb):
-            q, k = map(self.rotary_emb.rotate_queries_or_keys, (q, k))
-
         # add null key / value for classifier free guidance in prior net
 
         nk, nv = repeat_many(self.null_kv.unbind(dim = -2), 'd -> b 1 d', b = b)
@@ -388,16 +378,10 @@ class Attention(nn.Module):
             mask = rearrange(mask, 'b j -> b 1 1 j')
             sim = sim.masked_fill(~mask, max_neg_value)
 
-        if self.causal:
-            i, j = sim.shape[-2:]
-            causal_mask = torch.ones((i, j), dtype = torch.bool, device = device).triu(j - i + 1)
-            sim = sim.masked_fill(causal_mask, max_neg_value)
-
         # attention
 
         sim = sim - sim.amax(dim = -1, keepdim = True).detach()
         attn = sim.softmax(dim = -1)
-        attn = self.dropout(attn)
 
         # aggregate values
 
@@ -406,53 +390,6 @@ class Attention(nn.Module):
         out = rearrange(out, 'b h n d -> b n (h d)')
         return self.to_out(out)
 
-class CausalTransformer(nn.Module):
-    def __init__(
-        self,
-        *,
-        dim,
-        depth,
-        dim_head = 64,
-        heads = 8,
-        ff_mult = 4,
-        norm_out = True,
-        attn_dropout = 0.,
-        ff_dropout = 0.,
-        final_proj = True,
-        normformer = False,
-        rotary_emb = True
-    ):
-        super().__init__()
-        self.rel_pos_bias = RelPosBias(heads = heads)
-
-        rotary_emb = RotaryEmbedding(dim = min(32, dim_head)) if rotary_emb else None
-
-        self.layers = nn.ModuleList([])
-        for _ in range(depth):
-            self.layers.append(nn.ModuleList([
-                Attention(dim = dim, causal = True, dim_head = dim_head, heads = heads, dropout = attn_dropout, rotary_emb = rotary_emb),
-                FeedForward(dim = dim, mult = ff_mult, dropout = ff_dropout, post_activation_norm = normformer)
-            ]))
-
-        self.norm = LayerNorm(dim) if norm_out else nn.Identity()  # unclear in paper whether they projected after the classic layer norm for the final denoised image embedding, or just had the transformer output it directly: plan on offering both options
-        self.project_out = nn.Linear(dim, dim, bias = False) if final_proj else nn.Identity()
-
-    def forward(
-        self,
-        x,
-        mask = None    # we will need a mask here, due to variable length of the text encodings - also offer dalle1 strategy with padding token embeddings
-    ):
-        n, device = x.shape[1], x.device
-
-        attn_bias = self.rel_pos_bias(n, n + 1, device = device)
-
-        for attn, ff in self.layers:
-            x = attn(x, mask = mask, attn_bias = attn_bias) + x
-            x = ff(x) + x
-
-        out = self.norm(x)
-        return self.project_out(out)
-
 # decoder
 
 def Upsample(dim):
@@ -556,7 +493,6 @@ class CrossAttention(nn.Module):
         context_dim = None,
         dim_head = 64,
         heads = 8,
-        dropout = 0.,
         norm_context = False
     ):
         super().__init__()
@@ -568,7 +504,6 @@ class CrossAttention(nn.Module):
 
         self.norm = LayerNorm(dim)
         self.norm_context = LayerNorm(context_dim) if norm_context else nn.Identity()
-        self.dropout = nn.Dropout(dropout)
 
         self.null_kv = nn.Parameter(torch.randn(2, dim_head))
         self.to_q = nn.Linear(dim, inner_dim, bias = False)
@@ -1295,6 +1230,7 @@ class Imagen(BaseGaussianDiffusion):
         text_embeds = None,
         batch_size = 1,
         cond_scale = 1.,
+        lowres_sample_noise_level = None,
         stop_at_unet_number = None
     ):
         device = next(self.parameters()).device
@@ -1313,7 +1249,8 @@ class Imagen(BaseGaussianDiffusion):
         is_cuda = next(self.parameters()).is_cuda
         device = next(self.parameters()).device
 
-        lowres_noise_times = torch.full((batch_size,), self.lowres_sample_noise_level, device = device, dtype = torch.long)
+        lowres_sample_noise_level = default(lowres_sample_noise_level, self.lowres_sample_noise_level)
+        lowres_noise_times = torch.full((batch_size,), lowres_sample_noise_level, device = device, dtype = torch.long)
 
         for unet_number, unet, channel, image_size, learned_variance in tqdm(zip(range(1, len(self.unets) + 1), self.unets, self.sample_channels, self.image_sizes, self.learned_variance)):
 
